'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _extends2 = require('babel-runtime/helpers/extends');

var _extends3 = _interopRequireDefault(_extends2);

var _promise = require('babel-runtime/core-js/promise');

var _promise2 = _interopRequireDefault(_promise);

exports.default = ssrComponentWithInitialProps;

var _react = require('react');

var _react2 = _interopRequireDefault(_react);

var _server = require('react-dom/server');

var _server2 = _interopRequireDefault(_server);

var _withContext = require('@s-ui/hoc/lib/withContext');

var _withContext2 = _interopRequireDefault(_withContext);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var EMPTY_FUNC = function EMPTY_FUNC() {
  return _promise2.default.resolve({});
};

var hrTimeToMs = function hrTimeToMs(diff) {
  return diff[0] * 1e3 + diff[1] * 1e-6;
};

function ssrComponentWithInitialProps(_ref) {
  var Target = _ref.Target,
      context = _ref.context,
      renderProps = _ref.renderProps,
      _ref$useStream = _ref.useStream,
      useStream = _ref$useStream === undefined ? false : _ref$useStream;

  var startGetInitialProps = process.hrtime();
  // get the page with the initialProps
  var pageComponent = renderProps.components[renderProps.components.length - 1];
  // use the getInitialProps from the page to retrieve the props to initialize
  var getInitialProps = pageComponent.getInitialProps || EMPTY_FUNC;
  return getInitialProps(context).then(function (initialProps) {
    var diffGetInitialProps = process.hrtime(startGetInitialProps);
    // create the App component with the context, the initialProps and the Target Component
    var App = (0, _withContext2.default)((0, _extends3.default)({}, context, { initialProps: initialProps }))(Target);
    if (useStream) {
      var reactStream = _server2.default.renderToNodeStream(_react2.default.createElement(App, (0, _extends3.default)({}, renderProps, { initialProps: initialProps })));
      return {
        initialProps: initialProps,
        reactStream: reactStream,
        performance: {
          getInitialProps: hrTimeToMs(diffGetInitialProps)
        }
      };
    } else {
      // start to calculate renderToString
      var startRenderToString = process.hrtime();
      // render our whole application with the needed props and get the html string
      var reactString = _server2.default.renderToString(_react2.default.createElement(App, (0, _extends3.default)({}, renderProps, { initialProps: initialProps })));
      var diffRenderToString = process.hrtime(startRenderToString);
      return {
        initialProps: initialProps,
        reactString: reactString,
        performance: {
          getInitialProps: hrTimeToMs(diffGetInitialProps),
          renderToString: hrTimeToMs(diffRenderToString)
        }
      };
    }
  });
}